package edu.kit.ipd.descartes.adaptation.model.perfdatarepo;

import static edu.kit.ipd.descartes.adaptation.model.perfdatarepo.PerfDataRepoConverter.convertObservableImpact;

import org.apache.log4j.Logger;
import org.eclipse.emf.common.util.URI;

import edu.kit.ipd.descartes.adaptation.TacticsHistory;
import edu.kit.ipd.descartes.adaptation.model.AbstractEcoreModelLoader;
import edu.kit.ipd.descartes.mm.adaptation.WeightedTactic;
import edu.kit.ipd.descartes.perfdatarepo.Impact;
import edu.kit.ipd.descartes.perfdatarepo.Result;
import edu.kit.ipd.descartes.ql.models.repository.Observation;
import edu.kit.ipd.descartes.ql.models.repository.ObservationRepository;

public class ObservationRepositoryHandler implements IPerformanceDataRepositoryHandler {

    private static Logger logger = Logger.getLogger(ObservationRepositoryHandler.class);
    private AbstractEcoreModelLoader modelLoader = null;
    private ObservationRepository repository = null;
    private int iteration = 1;

    public ObservationRepositoryHandler() {
        modelLoader = ObservationRepositoryModelLoader.getInstance();
        logger.debug("ObservationRepositoryHandler loaded.");
    }

    @Override
    public Impact getLatestImpact() {
        return convertObservableImpact(repository.getLatestImpact());
    }

    @Override
    public void load(String modelFileUri) {
        repository = (ObservationRepository) modelLoader.load(URI.createFileURI(modelFileUri));
    }

    @Override
    public Result getCurrentSystemState() {
        WeightedTactic latestTactic = TacticsHistory.getInstance().getLatestAppliedTactic();
        Observation o = null;
        
        if (latestTactic == null) {
            o = repository.getLatestObservation();
        } else {
            String expName = ObservationRepositoryHelper.deriveExperimentName(latestTactic, iteration);
            logger.debug("Searching for result of " + expName);
            o = ObservationRepositoryHelper.getResultEndingWith(expName, repository);
        }

        return PerfDataRepoConverter.convertResult(o);
        // return convertResult(repository.getObservations().get(iteration++));
    }

    @Override
    public Impact getLatestImpact(WeightedTactic tactic) {
        String expName = ObservationRepositoryHelper.deriveExperimentName(tactic, iteration + 1);
        logger.debug("Searching for impact of " + expName);
        Impact latestImpact = PerfDataRepoConverter.convertObservableImpact(ObservationRepositoryHelper
                .searchImpactOfTactic(expName, repository));
        iteration++;
        return latestImpact;
    }
}
