package edu.kit.ipd.descartes.adaptation.model.perfdatarepo;

import java.util.List;

import org.eclipse.emf.ecore.util.EcoreUtil;

import edu.kit.ipd.descartes.mm.adaptation.WeightedTactic;
import edu.kit.ipd.descartes.ql.models.repository.ObservableImpact;
import edu.kit.ipd.descartes.ql.models.repository.Observation;
import edu.kit.ipd.descartes.ql.models.repository.ObservationRepository;

public class ObservationRepositoryHelper {

    public static ObservableImpact searchImpactOfTactic(String name, ObservationRepository repository) {
        // Get correct result
        Observation o = getResultEndingWith(name, repository);

        // Derive impact from result
        ObservableImpact i = getImpactForObservation(o, repository);
        return i;
    }

    public static String deriveExperimentName(WeightedTactic tactic) {
        return tactic.getName();
    }

    public static String deriveExperimentName(WeightedTactic tactic, int iteration) {

        String execTacticName = tactic.getName();

        if (execTacticName.contains("Low")) {
            String desc1 = "Xreq-par-Yps-desc4-gw-desc3-db-desc2-predict-Z ps=$-req=5-size=50000";
            return null;
        } else if (execTacticName.contains("High")) {
            String desc4 = "Xreq-par-Yps-desc4-gw-desc3-db-desc2-predict-Z ps=$-req=5-size=50000";
            return desc4.replace("$", Integer.toString(iteration));
        }
        return null;
    }

    public static ObservableImpact getImpactForObservation(Observation o, ObservationRepository repo) {
        if (o == null || repo == null)
            return null;
        
        List<ObservableImpact> impacts = repo.getImpacts();
        for (ObservableImpact oi : impacts) {
            if (EcoreUtil.equals(oi.getAfter(), o))
                return oi;
        }

        return null;
    }

    /**
     * Searches the observation repository for an observation that ends with the given suffix.
     * 
     * @param suffix
     * @return first result that ends the given suffix, null otherwise.
     */
    public static Observation getResultEndingWith(String suffix, ObservationRepository repo) {
        if (suffix == null || repo == null)
            return null;
        
        List<Observation> results = repo.getObservations();
        for (Observation o : results) {
            if (o.getIdentifier().endsWith(suffix))
                return o;
        }
        return null;
    }
}
